#!/usr/bin/env python
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright
# holders.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Channels definitions table update script
#
# Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
# Version info: $Id: update_channels.py,v 1.2 2008/08/01 15:00:03 ptedesco Exp $
# Release: $Name:  $

"""Update script for the channels definition table.

Starting from org.glite.data.transfer-agents version 3.0.3.1, transfer 
parameters are read from the t_channel table in the database, and not from
channel agents configuration files.

This script copies in the database the parameters defined in the channel agents
configuration files.

The script requires no parameters. You can run it simply with:
    python update_channels.py

The script will:
    - locate and read the channel agents configuration files.
    - generate an sql update script for the t_channel table.
    - run the sql script.

The following output files are produced:
    update_channels.log
        The log file of the script (same contents as standard output).
    update_channels.oracle.log
        The output of the sqlplus command
    update_channels.sql
        The sql update script
"""

import libxml2
import sys
import os
import os.path
import re

log_file = None

def open_log_file():
    global log_file
    if log_file is None:
        log_file = open('update_channels.log','w')

def close_log_file():
    global log_file
    if not log_file is None:
        log_file.close()
        log_file = None
    
def log(msg):
    global log_file
    print msg
    if not log_file is None:
        log_file.write(msg)
        log_file.write('\n')

class ScriptException (Exception):
    """Base class for exceptions in this script."""
    def __init__(self,msg):
        self.__msg = msg
    def __str__(self):
        return self.__msg


class ChannelAgentConfig:
    """Read a channel agent configuration file."""

    def __init__(self,channelName,channelType,configFilePath):
        """Constructor
            channelName: name of the channel
            configFilePath: path of the configuration file
        """
        self.ChannelName = channelName
        self.ChannelType = channelType
        log ('Reading %s channel %s configuration' % (self.ChannelType,self.ChannelName))
        doc = None
        try:
            doc = libxml2.parseFile(configFilePath)
            cfgNodes = doc.xpathEval('/service/components/component[@name="transfer-agent-ts-urlcopy"]')
            if 1 != len(cfgNodes):
                raise ScriptException('Configuration file [%s] does not contain transfer-agent-ts-urlcopy library configuration node' % (configFilePath))
            cfg = cfgNodes[0]
            # init params
            self.TransferTimeout = self._getInitParamValue(cfg,'TransferTimeout','1800')
            self.HttpTimeout = self._getInitParamValue(cfg,'HttpTimeout','40')
            self.SrmPutTimeout = self._getInitParamValue(cfg,'SrmPutTimeout','180')
            self.SrmGetTimeout = self._getInitParamValue(cfg,'SrmGetTimeout','180')
            self.SrmPutDoneTimeout = self._getInitParamValue(cfg,'SrmPutDoneTimeout','180')
            self.SrmGetDoneTimeout = self._getInitParamValue(cfg,'SrmGetDoneTimeout','180')
            self.TransferMarkersTimeout = self._getInitParamValue(cfg,'TransferMarkersTimeout','120')
            self.TcpBufferSize = self._getInitParamValue(cfg,'TcpBufferSize','0')
            self.BlockSize = self._getInitParamValue(cfg,'BlockSize','0')
            self.SrmStatusTimeout = self._getInitParamValue(cfg,'SrmStatusTimeout','180')
            self.SrmCopyDirection = self._getInitParamValue(cfg,'SrmCopyDirection','PULL')
            # config params
            log_level = self._getConfigParamValue(cfg,'LogLevel')
            verbose_log = self._getConfigParamValue(cfg,'VerboseLog')
            if (None != verbose_log and 'true' == verbose_log) or (None != log_level and 'DEBUG' == log_level):
                self.LogLevel = 'DEBUG'
            else:
                self.LogLevel = 'INFO'
            log ('Channel log level set to %s' % (self.LogLevel))

            # read DB connection parameters
            dbConnNodes = doc.xpathEval('/service/components/component[@name="agents-dao-oracle"]')
            if 1 != len(dbConnNodes):
                raise ScriptException('Configuration file [%s] does not contain agents-dao-oracle library configuration node' % (configFilePath))
            dbConn = dbConnNodes[0]
            self.ConnectString = self._getInitParamValue(dbConn,'ConnectString',None,False)
            self.User = self._getInitParamValue(dbConn,'User',None,False)
            self.Password = self._getInitParamValue(dbConn,'Password',None,False)
            if self.ConnectString is None or self.User is None or self.Password is None:
                raise ScriptException('Error reading database connection parameters from file [%s]' % (configFilePath))
        finally:
            if not doc is None:
                doc.freeDoc()

    def channelUpdateStatement(self):
        """Create the SQL update statement for the channel properties.
            Returns SQL statement string.
        """
        predicates = []
        def add_pred(colName,value):
            predicates.append("%s='%s'" % (colName,value))
        add_pred('urlcopy_tx_to',self.TransferTimeout)
        add_pred('srmcopy_to',self.TransferTimeout)
        add_pred('http_to',self.HttpTimeout)
        add_pred('urlcopy_put_to',self.SrmPutTimeout)
        add_pred('urlcopy_get_to',self.SrmGetTimeout)
        add_pred('urlcopy_putdone_to',self.SrmPutDoneTimeout)
        add_pred('urlcopy_getdone_to',self.SrmGetDoneTimeout)
        add_pred('urlcopy_txmarks_to',self.TransferMarkersTimeout)
        add_pred('tcp_buffer_size',self.TcpBufferSize)
        add_pred('blocksize',self.BlockSize)
        add_pred('srmcopy_refresh_to',self.SrmStatusTimeout)
        add_pred('srmcopy_direction',self.SrmCopyDirection)
        add_pred('tx_loglevel',self.LogLevel)
        statement = "UPDATE t_channel SET %s WHERE channel_name = '%s';" % (', '.join(predicates),self.ChannelName)
        return statement

    def _getInitParamValue(self,cfg,paramName,defaultValue = None,log_value = True):
        return self._getParamValue(cfg,'init',paramName,defaultValue,log_value)

    def _getConfigParamValue(self,cfg,paramName,defaultValue = None,log_value = True):
        return self._getParamValue(cfg,'config',paramName,defaultValue,log_value)

    def _getParamValue(self,cfg,paramType,paramName,defaultValue = None,log_value = True):
        """Get value of given parameter
            paramType: init or config
            paramName: parameter name
            defaultValue: value to return if node not found
            returns parameter value if found, defaultValue otherwise
        """
        xpath = '%s/param[@name="%s"][1]/value[1]' % (paramType,paramName)
        paramValueNodes = cfg.xpathEval(xpath)
        if 0 == len(paramValueNodes):
            log ('Value for %s param %s not found, returning default value %s' % (paramType,paramName,str(defaultValue)))
            return defaultValue
        value = paramValueNodes[0].content
        if log_value:
            log ('Value for %s param %s is %s' % (paramType,paramName,value))
        else:
            log ('Value for %s param %s read from configuration file' % (paramType,paramName))
        return value


def getChannelAgentConfigs(cfgFilesPath):
    """Get a list of channel agent configuration objects from files found under given directory.
        cfgFilesPath: configuration files path
    """
    channelNames = []
    channelAgentConfigs = []
    regexp = re.compile('glite-transfer-channel-agent-(?P<mode>\w+)-(?P<ch_name>[\w-]+)\.properties\.xml')
    for file in os.listdir(cfgFilesPath):
        file_path = os.path.join(cfgFilesPath,file)
        if not os.path.isfile(file_path):
            log ('Skipping [%s] (not a file)' % (file_path))
            continue
        match = regexp.match(file)
        if None == match:
            log ('Skipping [%s] (not a channel agent configuration file)' % (file_path))
            continue
        mode = match.group('mode')
        name = match.group('ch_name')
        # Verify if a configuration file for channel with given name was already found.
        # This might happen for example if a channel type was changed, so there is a config
        # file for the urlcopy agent and one for the srmcopy agent.
        if name in channelNames:
            raise ScriptException('More than one configuration file found for channel %s' % (name))
        channelNames.append(name)
        channelAgentConfigs.append(ChannelAgentConfig(name,mode,file_path))
    return channelAgentConfigs

def createSqlScript(path,configs):
    """Create the update sql script.
        path: the output script path.
        configs: array of ChannelAgentConfig objects.
    """
    log('Creating sql update script')
    sql_file = None
    try:
        sql_file = open('update_channels.sql','w')
        def writeline(str):
            sql_file.write(str)
            sql_file.write('\n')
        writeline('WHENEVER SQLERROR exit failure;')
        writeline('DECLARE')
        writeline('    ch_test VARCHAR(1024);')
        writeline('    ch_name VARCHAR(1024);')
        writeline('BEGIN')
        for config in configs:
            writeline("    ch_name := '%s';" % (config.ChannelName))
            writeline("    SELECT channel_name INTO ch_test FROM t_channel WHERE channel_name = '%s';" % (config.ChannelName))
            sql_file.write('    ')
            writeline(config.channelUpdateStatement())
        writeline('    COMMIT;')
        writeline('EXCEPTION')
        writeline('WHEN NO_DATA_FOUND THEN')
        writeline("raise_application_error(-20001,'Channel ' || ch_name || ' is not defined in the database.');")
        writeline('END;')
        writeline('/')
    finally:
        if None != sql_file:
            sql_file.close()
    log('Sql update script created successfully.')

def main(argv=None):
    retval = 0
    if argv is None:
        argv = sys.argv
    try:
        open_log_file()
        # get the configuration files directory
        if not os.environ.has_key('GLITE_LOCATION'):
            raise ScriptException('GLITE_LOCATION is not defined.')
        base_path = os.path.expandvars('${GLITE_LOCATION}/etc/glite-data-transfer-agents.d')
        # read the configuration files
        configs = getChannelAgentConfigs(base_path)
        if 0 == len(configs):
            raise ScriptException('No configuration files found')
        # create the update sql script
        sql_script_file_name = 'update_channels.sql'
        createSqlScript(sql_script_file_name,configs)
        # run the sqlplus command
        user_name = configs[0].User
        password = configs[0].Password
        connect_string = configs[0].ConnectString
        command = 'sqlplus %s/%s@"%s" < %s 2>&1 > update_channels.oracle.log' % (user_name,password,connect_string,sql_script_file_name)
        log ('Running sqlplus command')
        ret = os.system(command)
        if 0 == ret:
            log ("Update script executed successfully.")
        else:
            log ("SQL command execution failed.")
            retval = 3
            
    except ScriptException,exc:
        log ('Error executing the migration script :')
        log (exc)
        retval = 1
    except Exception,exc:
        log (("Unexpected exception : " + str(type(exc)) + " args = " + str(exc.args)))
        retval = 2

    close_log_file()
    return retval

if __name__ == '__main__':
    sys.exit(main())
