/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ChannelCache class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ChannelCache.h,v 1.3 2007/03/21 14:15:50 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTIONS_VO_CHANNELCACHE_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTIONS_VO_CHANNELCACHE_H_

#include "glite/data/transfer/agent/model/Channel.h"

#include <vector>
#include <string>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {

/**
 * Forward declaration of ChannelCache Implementation
 */
class ChannelCacheImpl;
    
/**
 * ChannelCache
 */
class ChannelCache : boost::noncopyable {
public:

    typedef boost::shared_ptr<model::Channel>  ChannelPtr; 

    /**
     * Constructor
     */
    ChannelCache();
        
    /**
     * Destructor
     */
    ~ChannelCache();

    /**
     * Set the Validity of an entry in the Cache
     */
    void ttl(unsigned int v);

    /**
     * Set the min delay beween two purge
     */
    void purgeDelay(unsigned int d);

    /**
     * Set Flag
     */
    bool isSet() const {return m_isSet;}
    
    /**
     * Insert a Channel into the Cache
     */
    void insert(const std::string& ssite, const std::string& dsite,ChannelPtr c) /*throw (LogicError)*/;

    /**
     * Get a Channel cache entry
     */
    ChannelPtr get(const std::string& channel_name) const /*throw (RuntimeError)*/;

    /**
     * Get a Channel cache entry (if exists)
     */
    ChannelPtr getFromSites(const std::string& ssite, const std::string& dsite) const /*throw (RuntimeError)*/;

    /**
     * Remove a Channel from the cache
     */
    void remove(const std::string& ssite, const std::string& dsite) /*throw ()*/;

    /**
     * Count the number of entries in the cache
     */
    unsigned int count() const /*throw ()*/;

    /**
     * Remember missing Channel
     */
    void rememberMissingChannel(const std::string& ssite, const std::string& dsite) /*throw ()*/;

    /**
     * Is Channel missing
     */
    bool isChannelMissing(const std::string& ssite, const std::string& dsite) const /*throw ()*/;

    /**
     * remove missing Channel
     */
    void removeMissingChannel(const std::string& ssite, const std::string& dsite) /*throw ()*/;

    /**
     * Remember site's groups
     */
    void rememberSiteGroups(const std::string& site, const std::vector<std::string>& groups) /*throw ()*/;
    /**
     * Remember site's group. If a site has no group, reconrd it with a empty group name
     */
    void rememberSiteGroup(const std::string& site, const std::string& group) /*throw ()*/;
    /**
     * Get site groups. If no site is recorded, throws a RuntimeError. If site 
     * has no groups, the groups array will be empty
     */
    void getSiteGroups(const std::string& site, std::vector<std::string>& groups) const /*throw (RuntimeError)*/;
    /**
     * Remove Site Groups
     */
    void removeSiteGroups(const std::string& site, const std::vector<std::string>& groups) /*throw ()*/;
    /**
     * Remove Site Group
     */
    void removeSiteGroup(const std::string& site, const std::string& group) /*throw ()*/;

    /**
     * clear the Cache
     */
    void clear() /*throw ()*/;

    /**
     * Purge invalid entries
     * @param delay [IN] falg to specify if the purge operation should be skipped 
     *    if the previous purge was issued before the interval specified by 
     *    the &quot;purgeDelay&quot; property. 
     */
    void purge(bool use_delay = true);

private:
    /**
     * Pimpl
     */
    boost::scoped_ptr<class ChannelCacheImpl> m_impl;
    /**
     * Set Flag
     */
    bool m_isSet;
};

} // end namespace vo
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTIONS_VO_CHANNELCACHE_H_
