/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel.h,v 1.17 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_MODEL_CHANNEL_H_
#define GLITE_DATA_TRANSFER_AGENTS_MODEL_CHANNEL_H_

#include "glite/data/transfer/agent/model/Property.h"
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace model   {

#define CHANNEL_DEFAULTPREPARINGFILESRATIO 2.0
    
/**
 * Channel Class
 * @stereotype ValueObject
 */
struct Channel {

    typedef enum{ // Define values so they could combined in OR
        S_UNDEF        = 0x0000,
        S_ACTIVE       = 0x0001,
        S_DRAIN        = S_ACTIVE     << 1,  //0x0002
        S_INACTIVE     = S_DRAIN      << 1,  //0x0004
        S_STOPPED      = S_INACTIVE   << 1,  //0x0008
        S_HALTED       = S_STOPPED    << 1,  //0x0010
        S_ARCHIVED     = S_HALTED     << 1   //0x0020
    } State;

    typedef enum{
        VS_UNDEF    = 0x0000,
        VS_ON       = 0x0001,
        VS_OFF      = VS_ON   << 1,  //0x0002
        VS_PAUSE    = VS_OFF  << 1,  //0x0004
    } VOState;

    typedef enum{
        T_UNDEF         = 0x0000,
        T_DEDICATED     = 0x0001,
        T_NON_DEDICATED = T_DEDICATED   << 1,  //0x0002
    } Type;
    
    typedef enum{
        LL_UNDEF    = 0x0000,
        LL_DEBUG    = 0x0001,
        LL_INFO     = LL_DEBUG  << 1, //0x0002
        LL_WARN     = LL_INFO   << 1, //0x0004
        LL_ERROR    = LL_WARN   << 1  //0x0008
    } LogLevel;
    
    typedef enum{
        SCD_UNDEF   = 0x0000,
        SCD_PULL    = 0x0001,
        SCD_PUSH    = SCD_PULL << 1
    } SrmCopyDirection;
    
    /**
     * Default Constructor
     */
    Channel():state(S_UNDEF),files(0),streams(0),bandwidth(0.0),
            throughput(0.0),lastActive((time_t)-1),lastModification((time_t)-1),
            seLimit(0),type(T_DEDICATED),
            // new in 3.1.0
            httpTimeout(-1),txLoglevel(LL_INFO),urlcopyPutTimeout(-1),
            urlcopyPutdoneTimeout(-1),urlcopyGetTimeout(-1),urlcopyGetdoneTimeout(-1),
            urlcopyTxTimeout(-1),urlcopyTxmarksTimeout(-1),srmcopyDirection(SCD_PULL),
            srmcopyTimeout(-1),srmcopyRefreshTimeout(-1),
            // new in 3.2.0
            targetDirCheck(true),urlcopyFirstTxmarkTimeout(-1),
            // new in 3.3.0
            txTimeoutPerMb(0.0),noTransferActivityTimeout(-1),preparingFilesRatio(CHANNEL_DEFAULTPREPARINGFILESRATIO),
            transferType("")
    {}

    /**
     * Constructor
     */
    Channel(
        const std::string&  n,                      // channel_name
        State               s,                      // channel_state
        const std::string&  ssite,                  // source_site
        const std::string&  dsite,                  // dest_site
        unsigned int        files,                  // num_of_files
        unsigned int        strms   = 0,            // num_of_streams
        const std::string&  tbs     = "",           // tcp_buffer_size
        double              b       = 0,            // bandwidth
        double              t       = 0,            // nominal_throughput
        const std::string&  c       = "",           // contact
        const std::string&  msg     = "",           // message
        const std::string&  a_dn    = "",           // admin_dn
        time_t              l       = (time_t)-1,   // last_active
        time_t              lm      = (time_t)-1,   // last_modification
        unsigned int        sel     = 0,            // se_limit
        Type                ct      = T_DEDICATED,  // channel_type
        // new in 3.1.0
        const std::string&  bs      = "",           // block size
        int                 httpto  = -1,           // http timeout
        LogLevel            ll      = LL_INFO,      // log level
        int                 ucpto   = -1,           // urlcopy put timeout
        int                 ucpdto  = -1,           // urlcopy putdone timeout
        int                 ucgto   = -1,           // urlcopy get timeout
        int                 ucgdto  = -1,           // urlcopy getdone timeout
        int                 uctxto  = -1,           // urlcopy transfer timeout
        int                 uctmto  = -1,           // urlcopy transfer markers timeout
        SrmCopyDirection    scd     = SCD_PULL,     // srm copy direction
        int                 scpto   = -1,           // srmcopy timeout
        int                 scprto  = -1,           // srmcopy refresh timeout
        // new in 3.2.0
        bool                tdc     = true,         // target directory check
        int                 ucftmt  = -1,           // urlcopy first transfer marker timeout
        double              ttpmb   = 0.0,
        int                 ntat    = -1,
        double              pfr     = CHANNEL_DEFAULTPREPARINGFILESRATIO,
        const std::string&  txtype  = ""
           ):
            name(n),state(s),sourceSite(ssite),destSite(dsite),
            files(files),streams(strms),tcpBS(tbs), bandwidth(b),throughput(t),
            contact(c),message(msg),adminDn(a_dn),
            lastActive(l),lastModification(lm),
            seLimit(sel),type(ct),
            // new in 3.1.0
            blocksize(bs),httpTimeout(httpto),txLoglevel(ll),urlcopyPutTimeout(ucpto),
            urlcopyPutdoneTimeout(ucpdto),urlcopyGetTimeout(ucgto),urlcopyGetdoneTimeout(ucgdto),
            urlcopyTxTimeout(uctxto),urlcopyTxmarksTimeout(uctmto),srmcopyDirection(scd),
            srmcopyTimeout(scpto),srmcopyRefreshTimeout(scprto),
            // new in 3.2.0
            targetDirCheck(tdc),urlcopyFirstTxmarkTimeout(ucftmt),
            // new in 3.3.0
            txTimeoutPerMb(ttpmb),noTransferActivityTimeout(ntat),preparingFilesRatio(pfr),transferType(txtype)
        {}

    /**
     * Name Property
     */
    Property<std::string> const name;

    /**
     * State Property
     */
    Property<State> state;

    /**
     * SourceSite Property
     */
    Property<std::string> const sourceSite;

    /**
     * DestSite Property
     */
    Property<std::string> const destSite;

    /**
     * Number of Files Property
     */
    Property<unsigned int> const files;
        
    /**
     * Number of Streams Property
     */
    Property<unsigned int> const streams;

    /**
     * TCP Buffer Size Property
     */
    Property<std::string> const tcpBS;

    /**
     * Bandwidth Property
     */
    Property<double> const bandwidth;
        
    /**
     * Nominal Throughput Property
     */
    Property<double> const throughput;
    
    /**
     * Contact Property
     */
    Property<std::string> const contact;
    
    /**
     * Message Property
     */
    Property<std::string> const message;
    
    /**
     * AdminDN Property
     */
    Property<std::string> const adminDn;
    
    /**
     * LastActive Property
     */
    Property<time_t> lastActive;

    /**
     * Last Admin Modification Property
     */
    Property<time_t> const lastModification;

    /**
     * SE Limit Property
     */
    Property<unsigned int> const seLimit;

    /**
     * Channel Type Property
     */
    Property<Type> const type;
    
    //--------------------------------------------------------------------------
    // added with schema version 3.1.0
    //--------------------------------------------------------------------------
    
    /**
     * Block size property
     */
    Property<std::string> const blocksize;

    /**
     * HTTP Timeout Property
     */
    Property<int> const httpTimeout;
    
    /**
     * Tx Log Level Property
     */
    Property<LogLevel> const txLoglevel;
    
    /**
     * Url Copy Put Timeout Property
     */
    Property<int> const urlcopyPutTimeout;
    
    /**
     * Url Copy PutDone Timeout Property
     */
    Property<int> const urlcopyPutdoneTimeout;
    
    /**
     * Url Copy Get Timeout Property
     */
    Property<int> const urlcopyGetTimeout;

    /**
     * Url Copy GetDone Timeout Property
     */
    Property<int> const urlcopyGetdoneTimeout;

    /**
     * Url Copy Tx Timeout Property
     */
    Property<int> const urlcopyTxTimeout;

    /**
     * Url Copy Tx Markers Timeout Property
     */
    Property<int> const urlcopyTxmarksTimeout;

    /**
     * SRM Copy direction Property
     */
    Property<SrmCopyDirection> const srmcopyDirection;

    /**
     * SRM Copy Timeout Property
     */
    Property<int> const srmcopyTimeout;

    /**
     * SRM Copy Refresh Timeout Property
     */
    Property<int> const srmcopyRefreshTimeout;

    //--------------------------------------------------------------------------
    // added with schema version 3.2.0
    //--------------------------------------------------------------------------

    /**
     * Flag stating if target directory's existence must be ensured.
     */
    Property<bool> const targetDirCheck;
    
    /**
     * Url Copy First Tx Marker Timeout Property
     */
    Property<int> const urlcopyFirstTxmarkTimeout;

    //--------------------------------------------------------------------------
    // added with schema version 3.3.0
    //--------------------------------------------------------------------------
    /**
     * size-based transfer timeout, in seconds per MB
     */
    Property<double> const txTimeoutPerMb;
    
    /**
     * maximum interval with no activity detected during the transfer, in seconds
     */
    Property<int> const noTransferActivityTimeout;
    
    /**
     * ratio between number of files in the preparation phase and number of files
     */
    Property<double> const preparingFilesRatio;
    
    /**
     * Transfer type (URLCOPY, SRMCOPY, STUB)
     */
    Property<std::string> transferType;
    
public:
    /**
     * Reset All R/W Properties
     */
    void reset(){
        state.reset();
        lastActive.reset();
        transferType.reset();
    }
};

} // End namespace model
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_MODEL_CHANNEL_H_
