/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Error Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Error.h,v 1.8 2008/12/11 13:24:44 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_MODEL_ERROR_H_
#define GLITE_DATA_TRANSFER_AGENTS_MODEL_ERROR_H_

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace model   {
    
/**
 * Error
 */
class Error{
public:

    /**
     * Error Category Type
     */
    typedef enum { 
        UNDEF                   =  0,
        TRANSFER                =  1, // Transfer Error (if no more specific catch-all category is provided)
        SECURITY                =  2, // Security Error
        NETWORK                 =  3, // Network problem between two remote servers
        CONFIGURATION           =  4, // Configuration Problem
        CONSISTENCY             =  5, // Consistency Problem
        CATALOG                 =  6, // Generic Catalog Error
        USER                    =  7, // The user provided invalid data, e.g. invalid surl format
        FILE_EXISTS             =  8, // Destination file already exists
        INVALID_PATH            =  9, // Source file does not exist or is not possible to create destination directory path
        NO_SPACE_LEFT           = 10, // No space left on target device
        INVALID_SIZE            = 11, // Inconsistent file size at source and destination after transfer
        REQUEST_FAILURE         = 12, // Failed to invoke remote method (RPC failure), e.g. SOAP_FAULT returned
        CONNECTION              = 13, // Connection error in contacting remote server (SRM,GRIDFTP,catalog)
        HTTP_TIMEOUT            = 14, // HTTP timeout 
        REQUEST_TIMEOUT         = 15, // Timeout in completing asynchronous request
        TRANSFER_TIMEOUT        = 16, // The transfer didn't complete in the expected time
        INTERNAL_ERROR          = 17, // Agent internal error, e.g. Logic error in the agent
        ABORTED                 = 18, // The request has been aborted
        GENERAL_FAILURE         = 19, // Generic error, catch-all category
        LOST                    = 20, // The transfer was lost (dead, killed)
        STORAGE_INTERNAL_ERROR  = 21, // Storage Element internal error
        GRIDFTP                 = 22, // Griftp failure
        MARKERS_TIMEOUT         = 23, // Transfer Markers Timeout
        SRM_COPY                = 24, // SrmCopy failure (catch-all category for srmcopy transfer errors)
        CATALOG_INTERNAL_ERROR  = 25, // catalog error catch-all category
        LOCALITY                = 26, // The transfer failed because of file location
        FIRST_MARKER_TIMEOUT    = 27, // First transfer marker timeout
        NO_PROGRESS             = 28, // No transfer activity timeout
        //----------------------------------------------------------------------
        // Deprecated
        // The following codes have been deprecated, either because they're now
        // mapped to a different code or because of the introduction of Scope 
        // and Phase.
        //----------------------------------------------------------------------
        AGENT                   = -1,  // Generic Agent Error [now INTERNAL_ERROR]
        SOURCE                  = -2,  // Generic problem with the Source SE
        SOURCE_PREP             = -3,  // Problem with the file preparation on the Source SE
        SOURCE_FINAL            = -4,  // Problem with the file finalization on the Source SE
        DESTINATION             = -5,  // Generic problem with the Destination SE
        DESTINATION_PREP        = -6,  // Problem with the file preparation on the Destination SE
        DESTINATION_FINAL       = -7,  // Problem with the file finalization on the Destination SE
        REMOTE                  = -8,  // Generic problem in contacting a remote server [now CONNECTION]
        CATALOG_SRC             = -9,  // Generic Catalog problem for the source file
        CATALOG_SRC_RES         = -10, // Problem during Source Name Resolution
        CATALOG_DST             = -11, // Generic Catalog problem for the destination file
        CATALOG_DST_RES         = -12, // Problem during Destination Name Resolution
        CATALOG_DST_REG         = -13  // Problem during Registration to the Destination Catalog
    } Category;

    /**
     * Error Scope
     */
    typedef enum {
        S_UNDEF       = 0,
        S_SOURCE      = 1, // source remote server (SRM, gridftp, catalog)
        S_DESTINATION = 2, // destination remote server (SRM, gridftp, catalog)
        S_TRANSFER    = 3, // transfer
        S_AGENT       = 4  // internal agent problem (inconsistency, file system errors, etc)
    } Scope;

    /**
     * Error Phase
     */
    typedef enum {
        P_UNDEF                 = 0,
        P_TX_PREPARATION        = 1, // transfer preparation phase
        P_TX_FINALIZATION       = 2, // transfer finalization phase
        P_TRANSFER              = 3, // transfer phase
        P_ALLOCATION            = 4, // file allocation to channel phase
        P_CATALOG_RESOLUTION    = 5, // catalog resolution
        P_CATALOG_REGISTRATION  = 6, // catalog registration
        P_PRESTAGING            = 7, // prestaging phase
        P_FINALIZATION          = 8, // file finalization phase 
        P_WAITING               = 9, // used only for constistency errors during retry logic evaluation
        P_WAITING_CATALOG       = 10,// used only for constistency errors during catalog retry logic evaluation
        P_CANCELING             = 11,// error during cancelation
        P_TX_SERVICE            = 12,// error during interaction with transfer service (fetching file, checking state)
        P_PRE_TRANSFER          = 13,// error in pre transfer hook (VO-specific validation failed)
        P_POST_TRANSFER         = 14,// error in pre transfer hook (VO-specific validation failed)
        P_CHECK_READINESS       = 15
    } Phase;
    
protected:
    /**
     * Constructor. Not Implemented
     */
    Error();
    /**
     * Destructor. Not Implemented
     */
    ~Error();
};



} // end namespace model
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_MODEL_ERROR_H_
