/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: TransferService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>,
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: TransferService.h,v 1.19 2009/07/02 15:21:17 molnarzs Exp $
 *  Release: $Name:  $
 */

#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_

#include "glite/data/agents/glite-types.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Error.h"
#include "glite/data/transfer/agent/ts/TransferException.h"

#include <glite/data/transfer/urlcopy/details/SurlPair.h>

#include <vector>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

using ::glite::data::transfer::urlcopy::details::SurlPair;

/**
 * TransferService interface. This class defines the interface to integrate
 * different TransferService implementations (Adapter Pattern)
 * @interface
 */
class TransferService : boost::noncopyable {
public:
    /**
     * TransferRequest Struct
     */
    struct TransferRequest{
        typedef std::vector<SurlPair> SurlPairArray;

        SurlPairArray    surls;
        std::string      parameters;
        unsigned long    streams;
        unsigned long    tcpbs;
        unsigned long    blocksize;
        std::string      src_srm;
        std::string      src_srm_version;
        std::string      dest_srm;
        std::string      dest_srm_version;
        std::string      dest_token_descr;
        std::string      dest_space_token;
        bool             overwrite_flag;
        std::string      description;
        // new in 3.1.0
        int              http_timeout;              // http timeout
        glite::data::transfer::agent::model::Channel::LogLevel log_level; // log level
        int              urlcopy_put_timeout;       // urlcopy put timeout
        int              urlcopy_putdone_timeout;   // urlcopy putdone timeout
        int              urlcopy_get_timeout;       // urlcopy get timeout
        int              urlcopy_getdone_timeout;   // urlcopy getdone timeout
        int              urlcopy_tx_timeout;        // urlcopy transfer timeout
        int              urlcopy_tx_markers_timeout;// urlcopy transfer markers timeout
        glite::data::transfer::agent::model::Channel::SrmCopyDirection srmcopy_direction; // srm copy direction
        int              srmcopy_timeout;           // srmcopy timeout
        int              srmcopy_refresh_timeout;   // srmcopy refresh timeout
        // new in 3.2.0
        std::string      source_token_descr;        // source space token description
        std::string      source_space_token;        // source space token
        bool             enable_dir_check;          // enable/disable checks for existance of target directory
        int              urlcopy_first_tx_marker_timeout; // first transfer marker timeout
        // new in 3.3.0
        double           tx_timeout_per_mb;
        int              no_tx_activity_timeout;
        int              copy_pin_lifetime;
        bool             lan_connection;
        bool             fail_nearline;
        /**
         * Default Constructor
         */
        TransferRequest();
    };

    /**
     * TransferResponse Struct
     */
    struct TransferResponse{
        std::string               request_id;
        std::vector<unsigned int> transfer_ids;
    };

    /**
     * TransferObject Array Type
     */
    typedef std::vector<glite::data::transfer::agent::model::Transfer*> TransferInfoArray;

    /**
     * Destructor
     */
    virtual ~TransferService(){}

    /**
     * Return the TransferService type
     * @return the TransferService type
     */
    virtual std::string type() /*throw ()*/ = 0;

    /**
     * Return the type of the transfer handled by the service, AS IT MUST BE
     * WRITTEN IN THE DATABASE
     * @return TransferType for the t_channel table
     */
    virtual std::string transferType() = 0;

    /**
     * Return the name of the Channel the TransferService is responsible for
     * @return the name of the Channel the TransferService is responsible for
     * @throws LogicError in case the channel name is not set
     */
    virtual std::string channel() /*throw (LogicError)*/ = 0;

    /**
     * Get the number of max concurrent transfer requests the service can support
     * @return the number of max concurrent transfer requests the service can support
     */
    virtual unsigned long max() /*throw ()*/ = 0;

    /**
     * Get the number of max transfers that can be processed in the same request
     * @return number of max transfers that can be processed in the same request
     */
    virtual unsigned long maxRequestSize() /*throw ()*/ = 0;

    /**
     * Submit Transfers
     * @param req [INOUT] the request containing the list of SURLs pair to
     * transfer.
     * @param out [OUT] The result  object containing the request identifier and
     * @param credentials [IN] the credentials to be used to perform the action
     * a list of transfer identifier
     * @throws LogicError or TransferException in case of error
     */
    virtual void submit(const TransferRequest& req, TransferResponse& out, const std::string& credentials) /*throw (LogicError, TransferException)*/ = 0;

    /**
     * Cancel a Pending Transfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     */
    virtual void revoke(const std::string& requestId, const std::string& credentials) /*throw ()*/ = 0;

    /**
     * Abort a Pending Transfer request. This method is intended for performing
     * a best effort clean up when the user credentials are not available to
     * revoke the transfer
     * @param requestId [IN] the Identifier of the pending Transfer request
     */
    virtual void abort(const std::string& requestId) /*throw ()*/ = 0;

    /**
     * Get the status of a FileTransfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param objs [INOUT] the array that will containing the information for
     * each transfer involved on the request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError, TransferException in case of error
     */
    virtual void state(const std::string& requestId, TransferInfoArray& objs, const std::string& credentials) /*throw (LogicError, TransferException)*/ = 0;

    /**
     * Clean-up the information concerning a completed Transfer
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError in case the requested Transfer request
     * doesn't exist or is not completed
     */
    virtual void clean(const std::string& requestId, const std::string& credentials) /*throw (LogicError)*/ = 0;

    /**
     * Trace a Transfer
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param out [IN] the buffer where to store the transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError in case the requested Transfer request
     * doesn't exist or is not completed
     * @throws TransferException otherwise
     */
    virtual void trace(const std::string& requestId, std::ostream& out, const std::string& credentials) /*throw (LogicError, TransferException)*/ = 0;

    /**
     * Get the number of all the pending transfer requests on the channel
     * @return the number of all the pending transfer requests on the channel
     * @throws LogicError or TransferException in case of error
     */
    virtual unsigned long count() /*throw (LogicError, TransferException)*/ = 0;

    /**
     * Get the maximum number of allowed transfers in any state for the channel
     * @param channel_transfers [IN]
     * @param preparingFilesRatio [IN] ratio between number of files in preparation
     * phase and in the actual transfer phase configured for the channel
     * @return maximum number of allowed transfers in any state for the channel
     */
    virtual unsigned long getMaxChannelTransfers(
        unsigned long channel_transfers,
        double preparingFilesRatio) = 0;

    /**
     * Get the maximum number of allowed transfers in any state for a VO
     * @param vo_transfers [IN]
     * @param preparingFilesRatio [IN] ratio between number of files in preparation
     * phase and in the actual transfer phase configured for the channel
     * @return maximum number of allowed transfers in any state for a VO
     */
    virtual unsigned long getMaxVOTransfers(
        unsigned long vo_transfers,
        double preparingFilesRatio) = 0;

    /**
     * return true if transfers stop after the preparation phase to maintain constant
     * the number of transfers in the transfer phase
     * @return
     */
    virtual bool stopAfterPreparation() /* throw() */ = 0;

    /**
     * start the transfer phase for given transfer
     * @param transfer [IN] the transfer for which transfer phase must be started
     */
    virtual void startTransferPhase(
        const model::Transfer&  transfer) /*throw (LogicError)*/ = 0;

    /**
     * Get the transfer states that represent an active transfer
     * @return
     */
    virtual model::Transfer::State getActiveStates() /* throw() */ = 0;

    /**
     * Get the transfer states that represent a transfer that needs starting the
     * transfer phase
     * @return
     */
    virtual model::Transfer::State getStatesToStart() /* throw() */ = 0;

private:

    /** @link dependency */
    /*# File lnkFile; */

    /** @link dependency
     * @label throws*/
    /*# TransferException lnkTransferException; */
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_
