#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#  This script implements the basic catalog retry strategy
#
#  Authors: Paolo Badino <paolo.badino@cern.ch>
#  Version info: $Id: basic_catalog_retry.py,v 1.4 2008/11/13 10:43:55 ptedesco Exp $
#  Release: $Name:  $
#

import time
import glite.fts
import glite.data.agents.utils

#
# Declare Constants used in the script
#
MODULE_NAME      = "basic_catalog_retry"
RETRY_DELAY_TIME = 600 #10 minutes
MAX_FAILURES     = 3

#
# Declare the Retry Version supported by this script
#
def CatalogRetryVersion():
    return "1.0"

# Log Functions
VerboseDebug  = False
DebugPriority = glite.data.agents.utils.LogPriority.Debug
def m_log_debug(msg):
    global DebugPriority 
    glite.data.agents.utils.Log(MODULE_NAME,DebugPriority,msg)
    
def m_log_info(msg):
    glite.data.agents.utils.LogInfo(MODULE_NAME,msg)

def m_log_warn(msg):
    glite.data.agents.utils.LogWarn(MODULE_NAME,msg)

def m_log_error(msg):
    glite.data.agents.utils.LogError(MODULE_NAME,msg)

#
# Apply the Basic Catalog Retry Strategy to the given File
#
def CatalogRetry(job,files):
    
    global VerboseDebug
    
    result = glite.fts.CatalogRetryResult.Wait
    
    m_log_debug("Check if Job [%s] can be retried. It has %d file" % (job.id, len(files)))
    if(VerboseDebug == True):
        for file in files:
            m_log_debug("File [%s] needs to be evaluated" % (file.id))
    
    catalog_failures = 0
    last_active_time = -1
    has_remote_error = False
    for file in files:
        if(catalog_failures == 0):
            catalog_failures = file.catalogFailures
        elif(catalog_failures > file.catalogFailures):
            catalog_failures = file.catalogFailures
        
        if(last_active_time != -1):
            last_active_time = file.finishTime
        elif(file.finishTime != -1):
            #Take the file finish time
            if(file.finishTime > last_active_time):
                last_active_time = file.finishTime

        if(file.reasonClass == glite.fts.ErrorCategory.Connection) or \
            (file.reasonClass == glite.fts.ErrorCategory.Remote):
            has_remote_error = True
            
    if(has_remote_error == False):
        # No Remote Failures= > Put it in FAILED State
        m_log_info("Job [%s] has Non-Remote Catalog Failures and will be failed" % (job.id))
        result = glite.fts.CatalogRetryResult.Fail
    elif(catalog_failures >= MAX_FAILURES):
        # Too Many Failures = > Put it in FAILED State
        m_log_info("Too many catalog failures (%d) for Job [%s]: Move it to Failed" % (catalog_failures, job.id))
        result = glite.fts.CatalogRetryResult.Fail
    else:
        delay = RETRY_DELAY_TIME
        # Get Current Time
        current = time.time()
        # Check if it's time to retry the File
        if(last_active_time == -1):
            # Take the Job Submission Time
            m_log_debug("Catalog Failures: %d - Job Submission Time: %d" % (catalog_failures, job.submitTime))
            if(catalog_failures > 0):
                last_active_time = job.submitTime + (catalog_failures - 1) * delay;
            else:
                last_active_time = job.submitTime

        m_log_debug("Last Active Reference Time: %d - CurrentTime: %d" % (last_active_time,current))
        if((current > last_active_time) and ((current - last_active_time) >= delay)):
            # Reschedule the File
            m_log_info("Reschedule Job [%s]. Number of tries already performed: %d" % (job.id,catalog_failures))
            # Resubmit Job
            result = glite.fts.CatalogRetryResult.Retry
        else:
            # Elapsed time not sufficient
            m_log_debug("Job [%s] still in Waiting" % (job.id))

    # return result
    m_log_debug("Retry Result for Job [%s] is %s" % (job.id, result))
    return result;
